Unit HHG;

Interface

Uses VGA, VGA_A, LogFile;

Type
  TSign = Array[0..2] of Char;
  TArray = Array[0..64000] of Byte;
  PArray = ^TArray;

Const
  HHG_Sign   : TSign = 'HHG';

  HHG_Errors : Array[0..3] of String[40] =
  ('No error','File not found','Invalid file format','Invalid header');

  Max_Images   = 99;

Type
  THeader = Record
    Sign       : TSign;
    Version    : Word;
    Num_Images : Byte;
    Width      : Word;
    Height     : Word;
  end;

  TDumbData = Record
    Palette    : VGA_Palette;
    Movie      : ByteBool;
    Reserved   : Array[0..244] of Byte;
  end;

  PImageFile = ^TImageFile;
  TImageFile = Record
    Header : THeader;
    Images : Array[0..Max_Images] of PArray;
  end;

Var
  HHG_Image : TImageFile;

function HHG_Save(Name : String; var Image : TImageFile) : Byte;
function HHG_Load(Name : String; var Image : TImageFile) : Byte;
procedure HHG_FreeMem(var Image : TImageFile);
procedure HHG_Init(var Image : TImageFile; W, H : Word);
procedure HHG_Add(var Image : TImageFile; var Buf);
procedure HHG_Draw(aX,aY:Word;N:Byte);
procedure HHG_Draw_2(const F:TImageFile;aX,aY:Word;N:Byte);
procedure HHG_Draw_Trans(const F : TImageFile;aX,aY:Word;Num:Byte);

Implementation

procedure HHG_Draw(aX,aY:Word;N:Byte);
var
  Y      : Word;
  Ofs    : Word;
  DblOfs : Word;
  begin
    Ofs := 0;
    DblOfs := (aY shl 8) + (aY shl 6) + aX;
    For Y := 0 to HHG_Image.Header.Height - 1 do
      begin
        Move(HHG_Image.Images[N]^[Ofs], Double_Buffer^[DblOfs], HHG_Image.Header.Width);
        Inc(Ofs, HHG_Image.Header.Width);
        Inc(DblOfs, 320);
      end;
    end;

procedure HHG_Draw_2(const F:TImageFile;aX,aY:Word;N:Byte);
var
  Y      : Word;
  Ofs    : Word;
  DblOfs : Word;
  begin
    Ofs := 0;
    DblOfs := (aY shl 8) + (aY shl 6) + aX;
    For Y := 0 to F.Header.Height - 1 do
      begin
        Move(F.Images[N]^[Ofs], Double_Buffer^[DblOfs], F.Header.Width);
        Inc(Ofs, F.Header.Width);
        Inc(DblOfs, 320);
      end;
    end;

function HHG_Save(Name : String; var Image : TImageFile) : Byte;
var
  F    : File;
  I    : Byte;
  Size : Word;
  R    : Integer;
  DumbData : TDumbData;
  begin
    FillChar(DumbData, SizeOf(DumbData), 0);
    Assign(F, Name);
    {$I-}
    ReWrite(F, 1);
    R := IOResult;
    HHG_Save := R;
    if R <> 0 then Exit;
    With Image do
      begin
{        Header.Palette := VGA_Pal^;}
        Size := Header.Width * Header.Height;
        Header.Sign    := HHG_Sign;
        Header.Version := 1;
{        FillChar(Header.Reserved, SizeOf(Header.Reserved), 0);}
        BlockWrite(F, Header, SizeOf(Header));
        BlockWrite(F, DumbData, SizeOf(DumbData));
        if Header.Num_Images > 0 then
        For I := 0 to Header.Num_Images - 1 do
        BlockWrite(F, Images[I]^, Size);
      end;
    Close(F);
  end;

function HHG_Load(Name : String; var Image : TImageFile) : Byte;
var
  F    : File;
  I    : Byte;
  Size : Word;
  R    : Integer;
  DumbData : TDumbData;
  begin
    HHG_Load := 0;
    LOG_Write('HHG_LOAD ('+Name+') : ');
    Assign(F, Name);
    {$I-}
    ReSet(F, 1);
    R := IOResult;
    HHG_Load := R;
    if R <> 0 then
      begin
        LOG_WriteLn('error;');
        Exit;
      end
     else
    LOG_WriteLn('ok');
    With Image do
      begin
        BlockRead(F, Header, SizeOf(Header));
        BlockRead(F, DumbData, SizeOf(DumbData));
        Size := Header.Width * Header.Height;
        if Header.Sign <> HHG_Sign then
          begin
            HHG_Load := 3;
            Exit;
          end;
        if Header.Num_Images > 0 then
        For I := 0 to Header.Num_Images - 1 do
          begin
            if MemAvail < Size then
              begin
                VGA_Set_Mode($3);
                LOG_WriteLn('error');
                LOG_Close;
                WriteLn('HHG_Load : not enough memory (',Size,'/',MemAvail,' needed/free)');
                WriteLn('  file:',Name);
                Halt;
              end;
            GetMem(Images[I], Size);
            BlockRead(F, Images[I]^, Size);
          end;
      end;
    Close(F);
  end;

procedure HHG_FreeMem(var Image : TImageFile);
var
  I    : Byte;
  Size : Word;
  begin
    With Image do
      begin
        Size := Header.Width * Header.Height;
        if Header.Num_Images > 0 then
        For I := 0 to Header.Num_Images - 1 do
        if Images[I] <> nil then
        FreeMem(Images[I], Size);
      end;
    end;

procedure HHG_Init(var Image : TImageFile; W, H : Word);
  begin
    With Image.Header do
      begin
        Num_Images := 0;
        Width  := W;
        Height := H;
      end;
    end;

procedure HHG_Add(var Image : TImageFile; var Buf);
var Size : Word;
  begin
    With Image do
      begin
        Size := Header.Width * Header.Height;
        GetMem(Images[Header.Num_Images], Size);
        Move(Buf, Images[Header.Num_Images]^, Size);
        Inc(Header.Num_Images);
      end;
    end;

procedure HHG_Draw_Trans(const F : TImageFile;aX,aY:Word;Num:Byte);
var
  Work_Offset,Offset,X,Y:Word;
  Data:Byte;
  begin
    Work_Offset := 0;
    Offset := (aY shl 8) + (aY shl 6) + aX;
    With F do
    For Y:=0 to F.Header.Width-1 do
      begin
        For X:=0 to F.Header.Height-1 do
          begin
            Data := Images[Num]^[Work_Offset+X];
            if Data <> 0 then
            Double_Buffer^[Offset+X] := Data;
          end;
        Inc(Offset,320);
        Inc(Work_Offset,F.Header.Width);
      end;
    end;

end.
